package sun.security.jgss.wrapper;

import sun.security.util.DerInputStream;
import java.security.Provider;
import javax.security.auth.kerberos.ServicePermission;
import sun.security.krb5.Realm;
import java.io.IOException;
import sun.security.jgss.GSSExceptionImpl;
import sun.security.util.ObjectIdentifier;
import sun.security.util.DerOutputStream;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.GSSException;
import sun.security.jgss.GSSUtil;
import org.ietf.jgss.Oid;
import sun.security.jgss.spi.GSSNameSpi;

public class GSSNameElement implements GSSNameSpi
{
    long pName;
    private String printableName;
    private Oid printableType;
    private GSSLibStub cStub;
    static final GSSNameElement DEF_ACCEPTOR;
    
    private static Oid getNativeNameType(Oid obj, GSSLibStub instance) {
        if (GSSUtil.NT_GSS_KRB5_PRINCIPAL.equals(obj)) {
            Oid[] array = null;
            try {
                array = instance.inquireNamesForMech();
            }
            catch (GSSException ex) {
                if (ex.getMajor() == 2 && GSSUtil.isSpNegoMech(instance.getMech())) {
                    try {
                        instance = GSSLibStub.getInstance(GSSUtil.GSS_KRB5_MECH_OID);
                        array = instance.inquireNamesForMech();
                    }
                    catch (GSSException ex2) {
                        SunNativeProvider.debug("Name type list unavailable: " + ex2.getMajorString());
                    }
                }
                else {
                    SunNativeProvider.debug("Name type list unavailable: " + ex.getMajorString());
                }
            }
            if (array != null) {
                int i = 0;
                while (i < array.length) {
                    if (array[i].equals(obj)) {
                        return obj;
                    }
                    ++i;
                }
                SunNativeProvider.debug("Override " + obj + " with mechanism default(null)");
                return null;
            }
        }
        return obj;
    }
    
    private GSSNameElement() {
        this.pName = 0L;
        this.printableName = "<DEFAULT ACCEPTOR>";
    }
    
    GSSNameElement(long pName, GSSLibStub cStub) throws GSSException {
        this.pName = 0L;
        assert cStub != null;
        if (pName == 0L) {
            throw new GSSException(3);
        }
        this.pName = pName;
        this.cStub = cStub;
        this.setPrintables();
        return;
    }
    
    GSSNameElement(byte[] bytes, Oid nativeNameType, GSSLibStub cStub) throws GSSException {
        this.pName = 0L;
        assert cStub != null;
        if (bytes == null) {
            throw new GSSException(3);
        }
        this.cStub = cStub;
        if (nativeNameType != null) {
            nativeNameType = getNativeNameType(nativeNameType, cStub);
            if (GSSName.NT_EXPORT_NAME.equals(nativeNameType)) {
                DerOutputStream derOutputStream = new DerOutputStream();
                Oid mech = this.cStub.getMech();
                try {
                    derOutputStream.putOID(new ObjectIdentifier(mech.toString()));
                }
                catch (IOException cause) {
                    throw new GSSExceptionImpl(11, cause);
                }
                byte[] byteArray = derOutputStream.toByteArray();
                bytes = new byte[4 + byteArray.length + 4 + bytes.length];
                int n = 0;
                bytes[n++] = 4;
                bytes[n++] = 1;
                bytes[n++] = (byte)(byteArray.length >>> 8);
                bytes[n++] = (byte)byteArray.length;
                System.arraycopy(byteArray, 0, bytes, n, byteArray.length);
                int n2 = n + byteArray.length;
                bytes[n2++] = (byte)(bytes.length >>> 24);
                bytes[n2++] = (byte)(bytes.length >>> 16);
                bytes[n2++] = (byte)(bytes.length >>> 8);
                bytes[n2++] = (byte)bytes.length;
                System.arraycopy(bytes, 0, bytes, n2, bytes.length);
            }
        }
        this.pName = this.cStub.importName(bytes, nativeNameType);
        this.setPrintables();
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null && !Realm.AUTODEDUCEREALM) {
            String krbName = this.getKrbName();
            int lastIndex = krbName.lastIndexOf(64);
            if (lastIndex != -1) {
                String substring = krbName.substring(lastIndex);
                if ((nativeNameType != null && !nativeNameType.equals(GSSUtil.NT_GSS_KRB5_PRINCIPAL)) || !new String(bytes).endsWith(substring)) {
                    try {
                        securityManager.checkPermission(new ServicePermission(substring, "-"));
                    }
                    catch (SecurityException ex) {
                        throw new GSSException(11);
                    }
                }
            }
        }
        SunNativeProvider.debug("Imported " + this.printableName + " w/ type " + this.printableType);
        return;
    }
    
    private void setPrintables() throws GSSException {
        Object[] displayName = this.cStub.displayName(this.pName);
        assert displayName != null && displayName.length == 2;
        this.printableName = (String)displayName[0];
        assert this.printableName != null;
        this.printableType = (Oid)displayName[1];
        if (this.printableType == null) {
            this.printableType = GSSName.NT_USER_NAME;
        }
        return;
    }
    
    public String getKrbName() throws GSSException {
        GSSLibStub gssLibStub = this.cStub;
        if (!GSSUtil.isKerberosMech(this.cStub.getMech())) {
            gssLibStub = GSSLibStub.getInstance(GSSUtil.GSS_KRB5_MECH_OID);
        }
        long canonicalizeName = gssLibStub.canonicalizeName(this.pName);
        Object[] displayName = gssLibStub.displayName(canonicalizeName);
        gssLibStub.releaseName(canonicalizeName);
        SunNativeProvider.debug("Got kerberized name: " + displayName[0]);
        return (String)displayName[0];
    }
    
    @Override
    public Provider getProvider() {
        return SunNativeProvider.INSTANCE;
    }
    
    @Override
    public boolean equals(GSSNameSpi gssNameSpi) throws GSSException {
        return gssNameSpi instanceof GSSNameElement && this.cStub.compareName(this.pName, ((GSSNameElement)gssNameSpi).pName);
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof GSSNameElement)) {
            return false;
        }
        try {
            return this.equals((GSSNameElement)o);
        }
        catch (GSSException ex) {
            return false;
        }
    }
    
    @Override
    public int hashCode() {
        return new Long(this.pName).hashCode();
    }
    
    @Override
    public byte[] export() throws GSSException {
        byte[] exportName = this.cStub.exportName(this.pName);
        int offset = 0;
        if (exportName[offset++] != 4 || exportName[offset++] != 1) {
            throw new GSSException(3);
        }
        int len = (0xFF & exportName[offset++]) << 8 | (0xFF & exportName[offset++]);
        ObjectIdentifier objectIdentifier;
        try {
            objectIdentifier = new ObjectIdentifier(new DerInputStream(exportName, offset, len));
        }
        catch (IOException cause) {
            throw new GSSExceptionImpl(3, cause);
        }
        Oid oid = new Oid(objectIdentifier.toString());
        assert !(!oid.equals(this.getMechanism()));
        int n = offset + len;
        int n2 = (0xFF & exportName[n++]) << 24 | (0xFF & exportName[n++]) << 16 | (0xFF & exportName[n++]) << 8 | (0xFF & exportName[n++]);
        if (n2 < 0) {
            throw new GSSException(3);
        }
        byte[] array = new byte[n2];
        System.arraycopy(exportName, n, array, 0, n2);
        return array;
    }
    
    @Override
    public Oid getMechanism() {
        return this.cStub.getMech();
    }
    
    @Override
    public String toString() {
        return this.printableName;
    }
    
    @Override
    public Oid getStringNameType() {
        return this.printableType;
    }
    
    @Override
    public boolean isAnonymousName() {
        return GSSName.NT_ANONYMOUS.equals(this.printableType);
    }
    
    public void dispose() {
        if (this.pName != 0L) {
            this.cStub.releaseName(this.pName);
            this.pName = 0L;
        }
    }
    
    @Override
    protected void finalize() throws Throwable {
        this.dispose();
    }
    
    static {
        DEF_ACCEPTOR = new GSSNameElement();
    }
}
